// AO From Project Luma Shaders

vec3 toScreenSpace(vec2 p) {
	vec4 fragposition = gbufferProjectionInverse * vec4(vec3(p, texture2D(depthtex0, p).x) * 2.0 - 1.0, 1.0);
	return fragposition.xyz /= fragposition.w;
}

int bitfieldReverse(int a) {
	a = ((a & 0x55555555) << 1 ) | ((a & 0xAAAAAAAA) >> 1);
	a = ((a & 0x33333333) << 2 ) | ((a & 0xCCCCCCCC) >> 2);
	a = ((a & 0x0F0F0F0F) << 4 ) | ((a & 0xF0F0F0F0) >> 4);
	a = ((a & 0x00FF00FF) << 8 ) | ((a & 0xFF00FF00) >> 8);
	a = ((a & 0x0000FFFF) << 16) | ((a & 0xFFFF0000) >> 16);
		return a;
}

#define hammersley(i, N) vec2( float(i) / float(N), float( bitfieldReverse(i) ) * 2.3283064365386963e-10 )
#define tau 6.2831853071795864769252867665590
#define circlemap(p) (vec2(cos((p).y*tau), sin((p).y*tau)) * p.x)

float jaao(vec2 p, bool sky) {
// By Jodie.

	const float radius = 1.0;
	const int steps = 16;

	float ao = 1.0;

	vec3 p3 = toScreenSpace(p);
	vec3 normal = normalize(cross(dFdx(p3), dFdy(p3)));
	vec2 clipRadius = radius * vec2(viewHeight / viewWidth, 1.0) / length(p3);

	vec3 v = normalize(-p3);

	float nvisibility = 0.0;
	float vvisibility = 0.0;

	for (int i = 0; i < steps; i++) {

		vec2 circlePoint = circlemap(hammersley(i * 15 + 1, 16 * steps)) * clipRadius;
		circlePoint *= ditherGradNoise() + 0.1;

		vec3 o  = toScreenSpace(circlePoint    +p) - p3;
		vec3 o2 = toScreenSpace(circlePoint*.25+p) - p3;
		float l  = length(o );
		float l2 = length(o2);
		o /=l ;
		o2/=l2;

		nvisibility += clamp(1.-max(
			dot(o , normal) - clamp((l -radius)/radius,0.,1.),
			dot(o2, normal) - clamp((l2-radius)/radius,0.,1.)
		), 0., 1.);

		vvisibility += clamp(1.-max(
			dot(o , v) - clamp((l -radius)/radius,0.,1.),
			dot(o2, v) - clamp((l2-radius)/radius,0.,1.)
		), 0., 1.);

	}

	ao = min(vvisibility * 2.0, nvisibility) / float(steps);

	if (sky) ao = 1.0;

	return ao;
}