// Adjust Shader Setting
#define FILTERED_COLORED_SHADOWS

// Uniform
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;

#ifdef FILTERED_COLORED_SHADOWS
	uniform sampler2D shadowcolor0;
	uniform sampler2DShadow shadowtex0;
	uniform sampler2DShadow shadowtex1;
	
	const bool shadowHardwareFiltering0 = true;
	const bool shadowHardwareFiltering1 = true;
#else
	uniform sampler2D shadow;
#endif

// Include
#include "/lib/distort.glsl"

// Common Function
vec3 worldToShadow(vec4 worldPosition) {
  vec4 shadowPosition = shadowModelView * worldPosition;
  shadowPosition = shadowProjection * shadowPosition;
    
  shadowPosition.xy = distortShadow(shadowPosition.xy);
  shadowPosition /= shadowPosition.w;

  return vec3(shadowPosition * 0.5 + 0.5);
}

// Function
#ifdef FILTERED_COLORED_SHADOWS
float FilteredShadows(sampler2DShadow shadowTexture, vec3 shadowPosition) {
	float shadowDepth = 0.0005;
	float new_Pos_Z = shadowPosition.z - shadowDepth;
	
	return shadow2DLod(shadowTexture, vec3(shadowPosition.xy, new_Pos_Z), 0).z;
}

vec3 CalculateSunlightVisibility(vec3 shadowPosition, float dist, float NdotL) {
    float shadowMap0 = FilteredShadows(shadowtex0, shadowPosition);
    float shadowMap1 = FilteredShadows(shadowtex1, shadowPosition);

    vec4 shadowColor = texture2D(shadowcolor0, shadowPosition.xy);
    vec3 transmittedColor = shadowColor.rgb * 3.14 * (1.0 - shadowColor.a);
    
    vec3 shadows = mix(transmittedColor * shadowMap1, vec3(1.0,1.0,1.0), shadowMap0);
    shadows *= NdotL;
    shadows = max(shadows, vec3(extShadow));

    return clamp(1.0 - shadows, 0.0, 1.0);
}
  
#else

vec3 CalculateSunlightVisibility(vec3 shadowPosition, float dist, float NdotL) {      
    float shade = 0.0;
    float shadowDepth = texture2D(shadow, shadowPosition.xy).z;
    if (shadowDepth + 0.0005 < shadowPosition.z){
	     shade = 1.0;
    }
	
    shade -= clamp((dist - 0.7) * 5.0, 0.0, 1.0);
    shade = clamp(shade, 0.0, 1.0); 
    
    vec3 shadows = vec3(shade);
    shadows *= NdotL;
    shadows = max(shadows, vec3(extShadow));

    return shadows;
}
#endif