/*
CilokGaming Shaders
Newbie Player😅
*/

// Settings
#include "/lib/settings.glsl"

//Fragment Shader
#ifdef FSH

// Uniform
uniform sampler2D gcolor;
uniform sampler2D depthtex0;
uniform sampler2D composite;
uniform sampler2D gnormal;

uniform vec3 cameraPosition;
uniform vec3 previousCameraPosition;

uniform ivec2 eyeBrightnessSmooth;

uniform float centerDepthSmooth;
uniform float viewWidth;
uniform float viewHeight;
uniform float aspectRatio;
uniform float hujan;
uniform float far, near;

uniform int isEyeInWater;

uniform mat4 gbufferProjectionInverse;
uniform mat4 gbufferPreviousProjection;
uniform mat4 gbufferPreviousModelView;

uniform mat4 gbufferModelViewInverse;

// Varying
in vec2 texcoord;

in vec3 colorWaterMurk;
in vec3 colorWaterBlue;

// Common Function
float ditherGradNoise() {
	return fract(52.9829189 * fract(0.06711056 * gl_FragCoord.x + 0.00583715 * gl_FragCoord.y));
}

float GetDepthLinear(in vec2 coord) 
{					
	return (near * far) / (texture2D(depthtex0, coord).x * (near - far) + far);
}

vec3 Tonemap(vec3 color) {
	color = mix(color, vec3(dot(color, vec3(0.3333))), -1.0 * 1.15 + 1.0);
	color = pow(color, vec3(2.2));
	color = pow(color, vec3(1.1 * 1.0)) * 1.0;
	color *= 2.4 * 1.0;
	color = 1.0 - exp(-color);
	color = pow(color, vec3(0.4545));

	return color;
}

void CalculateUnderwaterFog(inout vec3 color) {
	vec3 fogColor = colorWaterMurk;
	float fogFactor = GetDepthLinear(texcoord) / 100.0f;
		  fogFactor = min(fogFactor, 0.7f);
		  fogFactor = sin(fogFactor * 3.1415 / 2.0f);
		  fogFactor = pow(fogFactor, 0.5f);

	color.rgb = mix(color.rgb, fogColor * 0.002f, vec3(fogFactor));
	color.rgb *= mix(vec3(1.0f), colorWaterBlue * colorWaterBlue * colorWaterBlue * colorWaterBlue, vec3(fogFactor));
}

vec3 ChromaticAberration()
{
    float distortion = 0.1;
    float edge = 4.0;

    float mask = length(texcoord.xy - 0.5);
    mask = pow(mask, edge);

    vec3 result = vec3(0.0);

    for(float i = 1.0; i <= CHROMATIC_ABERRATION_STRENGTH; i += 1.0 ){
        vec2 dir = normalize(texcoord.xy - 0.5);
        result.r += texture2DLod(gcolor, texcoord.xy - (dir * (i / CHROMATIC_ABERRATION_STRENGTH) * distortion * mask), 0.0).r;
        result.g += texture2DLod(gcolor, texcoord.xy - (dir * (i / CHROMATIC_ABERRATION_STRENGTH) * 0.66 * distortion * mask), 0.0).g;
        result.b += texture2DLod(gcolor, texcoord.xy - (dir * (i / CHROMATIC_ABERRATION_STRENGTH) * 0.33 * distortion * mask), 0.0).b;
    }

    result /= CHROMATIC_ABERRATION_STRENGTH;

    return result;
}

// Include
#include "/lib/DepthOfField.glsl"
#include "/lib/MotionBlur.glsl"
#include "/lib/LensFlare.glsl"

// Program
void main() {

	#ifdef CHROMATIC_ABERRATION
		vec3 color = ChromaticAberration();
	#else
		vec3 color = texture2D(gcolor, texcoord.st).rgb;
	#endif

	#ifdef MOTION_BLUR
		MotionBlur(color);
	#endif
	
	#ifdef DOF
		float raw_depth = texture2D(depthtex0, texcoord).r;
		float blur_factor = abs(pow(centerDepthSmooth, 75) - pow(raw_depth, 75));
		blur_factor = clamp(pow(1 - blur_factor, 3), 0, 1);

		if (blur_factor < 1)
		{
			color = mix(blur(gcolor, texcoord, (1 - blur_factor) * 3.5).rgb, color, blur_factor);
		}
	#endif
	
	#ifdef LENS_FLARE_SHADERS
		color = lensFlare(color, texcoord.st);
	#endif
	
	if (isEyeInWater == 1) {
		CalculateUnderwaterFog(color);
	}
	
	color = Tonemap(color);
	color += ditherGradNoise() / 255.0;

	gl_FragColor = vec4(color, 1.0);
}


#endif

//Vertex Shader
#ifdef VSH

// Uniform
uniform float viewWidth;
uniform float viewHeight;
uniform vec3 sunPosition;
uniform mat4 gbufferProjection;
uniform sampler2D depthtex0;
 
// Varying
varying float sunVisibility;

varying vec2 lf1Pos, lf2Pos, lf3Pos, lf4Pos, lf5Pos, lf6Pos, lf7Pos, lf8Pos, lf9Pos, lf10Pos, lf11Pos, lf12Pos, lf13Pos;
varying vec2 texcoord;

varying vec3 colorWaterMurk;
varying vec3 colorWaterBlue;

// Program
void main() {
	gl_Position = ftransform();	
	texcoord = gl_MultiTexCoord0.st;
	sunVisibility = 0.0f;
	
	vec4 ndcSunPosition = gbufferProjection * vec4(normalize(sunPosition), 1.0);
		ndcSunPosition /= ndcSunPosition.w;
		
	vec2 pixelSize = vec2(1.0 / viewWidth, 1.0 / viewHeight);
 
	vec2 screenSunPosition = vec2(-10.0);
	lf1Pos = lf2Pos = lf3Pos = lf4Pos = vec2(-10.0);
	
	if (ndcSunPosition.x >= -1.0 && ndcSunPosition.x <= 1.0 &&
		ndcSunPosition.y >= -1.0 && ndcSunPosition.y <= 1.0 &&
		ndcSunPosition.z >= -1.0 && ndcSunPosition.z <= 1.0)
	{
		screenSunPosition = ndcSunPosition.xy * 0.5 + 0.5;
		
		for(int x = -4; x <= 4; x++)
		{
			for(int y = -4; y <= 4; y++)
			{
				float depth = texture2DLod(depthtex0, screenSunPosition.st + vec2(float(x), float(y)) * pixelSize, 0.0).r;
					if (depth > 0.9999)
						sunVisibility += 1.0 / 81.0;
			}
		}
		
	float shortestDis = min( min(screenSunPosition.s, 1.0 - screenSunPosition.s),
										min(screenSunPosition.t, 1.0 - screenSunPosition.t));
										
	sunVisibility *= smoothstep(0.0, 0.2, clamp(shortestDis, 0.0, 0.2));
     
	vec2 dir = vec2(0.5) - screenSunPosition;
	lf1Pos = vec2(0.5) + dir * -00000001.00000001;
	lf2Pos = vec2(0.5) + dir * -00000001.00000001;
	lf3Pos = vec2(0.5) + dir * -00000001.00000001;
	lf4Pos = vec2(0.5) + dir * -00000001.00000001;
	lf5Pos = vec2(0.5) + dir * 0.99;
	lf6Pos = vec2(0.5) + dir * 0.49;
	lf7Pos = vec2(0.5) + dir * 0.39;
	lf8Pos = vec2(0.5) + dir * 0.45;
	lf9Pos = vec2(0.5) + dir * 0.20;
	lf10Pos = vec2(0.5) + dir * -0.10;
	lf11Pos = vec2(0.5) + dir * -0.19;
	lf12Pos = vec2(0.5) + dir * 0.20;
	lf13Pos = vec2(0.5) + dir * 0.40;

	}
	
	colorWaterMurk = vec3(0.2f, 0.5f, 0.95f);
	colorWaterBlue = vec3(0.2f, 0.5f, 0.95f);
	colorWaterBlue = mix(colorWaterBlue, vec3(1.0f), vec3(0.5f));
}

#endif