/*
CilokGaming Shaders
Newbie Player😅
*/

// Settings
#include "/lib/settings.glsl"

//Fragment Shader
#ifdef FSH

// Uniform
uniform sampler2D texture;
uniform sampler2D noisetex;
uniform sampler2D depthtex0;

uniform int worldTime;

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 gbufferProjection;
uniform mat4 gbufferProjectionInverse;

uniform float viewHeight;
uniform float viewWidth;
uniform float far, near;
uniform float frameTimeCounter;

uniform vec3 fogColor;
uniform vec3 skyColor;

uniform vec3 cameraPosition;

// Varying
in vec4 color;
in vec4 posisiMata;

in vec2 texcoord;
in vec2 lmcoord;

in vec3 binormal;
in vec3 normal;
in vec3 tangent;
in vec3 wPos;
in vec3 colorWaterMurk;
in vec3 colorWaterBlue;

in float iswater;

// Include
#include "/lib/deteksiWaktu.glsl"
#include "/lib/noise.glsl"
#include "/lib/langit.glsl"

// Common Variable
const float PI = 3.1415927;

// Common Function
float wave(float n) {
	return sin(2 * PI * (n));
}

float GetDepthLinear(in vec2 coord) 
{					
	return (near * far) / (texture2D(depthtex0, coord).x * (near - far) + far);
}

// Function
void CalculateUnderwaterFog(inout vec3 color) {
	vec3 fogColor = colorWaterMurk;
	float fogFactor = GetDepthLinear(texcoord) / 100.0f;
		  fogFactor = min(fogFactor, 0.7f);
		  fogFactor = sin(fogFactor * 3.1415 / 2.0f);
		  fogFactor = pow(fogFactor, 0.5f);

	color.rgb = mix(color.rgb, fogColor * 0.002f, vec3(fogFactor));
	color.rgb *= mix(vec3(1.0f), colorWaterBlue * colorWaterBlue * colorWaterBlue * colorWaterBlue, vec3(fogFactor));
}

float waterH(vec3 posxz) {
	float wave = 0.0;
	float factor = 2.0;
	float amplitude = 0.2;
	float speed = 2.0;
	float size = 0.2;

	float px = posxz.x / 50.0 + 250.0;
	float py = posxz.z / 50.0  + 250.0;

	float fpx = abs(fract(px * 20.0) - 0.5) * 2.0;
	float fpy = abs(fract(py * 20.0) - 0.5) * 2.0;

	float d = length(vec2(fpx, fpy));

	for (int i = 1; i < 4; i++) {
		wave -= d * factor * cos((1 / factor) * px * py * size + 1.0 * frameTimeCounter * speed);
		factor /= 2;
	}

	factor = 1.0;
	px = -posxz.x / 50.0 + 250.0;
	py = -posxz.z / 150.0 - 250.0;

	fpx = abs(fract(px * 20.0) - 0.5) * 2.0;
	fpy = abs(fract(py * 20.0) - 0.5) * 2.0;

	d = length(vec2(fpx, fpy));
	float wave2 = 0.0;
	for (int i = 1; i < 4; i++) {
		wave2 -= d * factor * cos((1 / factor) * px * py * size + 1.0 * frameTimeCounter * speed);
		factor /= 2;
	}
	return amplitude * wave2 + amplitude * wave;
}

// Program
void main() {
	vec4 tex = texture2D(texture, texcoord);
	
	vec3 ukuranLayar = vec3(gl_FragCoord.xy / vec2(viewWidth, viewHeight), gl_FragCoord.z);
	vec4 vektorRefleksi = gbufferProjectionInverse * vec4(ukuranLayar * 2.0 - 1.0, 1.0);
	vektorRefleksi /= vektorRefleksi.w;
	vec3 reflection = renderLangit(vektorRefleksi.xyz);
	
	vec3 direksiMata = normalize(-posisiMata.xyz);
	float NdotV = dot(normal, direksiMata);
	NdotV = clamp(NdotV, 0.0, 1.0);

	float reflectivity = 0.04;
	float fresnel = reflectivity + (1.0 - reflectivity) * pow(1.0 - NdotV, 5.0);

	tex = mix(vec4(0.172, 0.212, 0.54, 0.2), vec4(reflection, 1.0), fresnel);

	if (iswater < 0.1)  tex = texture2D(texture, texcoord.xy) * color;
	
	vec3 posxz = wPos.xyz;

	posxz.x += sin(posxz.z + frameTimeCounter) * 0.25;
	posxz.z += cos(posxz.x + frameTimeCounter * 0.5) * 0.25;
	
	float deltaPos = 0.4;
	float h0 = waterH(posxz);
	float h1 = waterH(posxz + vec3(deltaPos, 0.0, 0.0));
	float h2 = waterH(posxz + vec3(-deltaPos, 0.0, 0.0));
	float h3 = waterH(posxz + vec3(0.0, 0.0, deltaPos));
	float h4 = waterH(posxz + vec3(0.0, 0.0, -deltaPos));
	
	float xDelta = ((h1 - h0) + (h0 - h2)) / deltaPos;
	float yDelta = ((h3 - h0) + (h0 - h4)) / deltaPos;
	
	vec3 newnormal = normalize(vec3(xDelta, yDelta, 1.0 - xDelta * xDelta - yDelta * yDelta));
	
	vec4 frag2;
		frag2 = vec4((normal) * 0.5f + 0.5f, 1.0f);		
		
	if (iswater > 0.9) {
		vec3 bump = newnormal;
			bump = bump;
		float bumpmult = 0.05;	
		
		bump = bump * vec3(bumpmult, bumpmult, bumpmult) + vec3(0.0f, 0.0f, 1.0f - bumpmult);
		mat3 tbnMatrix = mat3(tangent.x, binormal.x, normal.x,
							tangent.y, binormal.y, normal.y,
							tangent.z, binormal.z, normal.z);
		
		frag2 = vec4(normalize(bump * tbnMatrix) * 0.5 + 0.5, 1.0);
	}
	
	vec3 color1 = vec3(tex);
	CalculateUnderwaterFog(color1);
	
	/* DRAWBUFFERS:0246 */
	gl_FragData[0] = tex;
	gl_FragData[1] = frag2;	
	gl_FragData[2] = vec4(lmcoord.t, mix(1.0, 0.05, iswater), lmcoord.s, 1.0);
	gl_FragData[3] = vec4(vec3(0.0), 1.0);
}

#endif

//Vertex Shader
#ifdef VSH

// Uniform
uniform vec3 cameraPosition;
uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform int worldTime;
uniform int moonPhase;
uniform float frameTimeCounter;
uniform float hujan;
uniform int isEyeInWater;

// Varying
varying vec4 color;
varying vec4 posisiMata;

varying vec2 texcoord;
varying vec2 lmcoord;

varying vec3 binormal;
varying vec3 normal;
varying vec3 tangent;
varying vec3 wPos;
varying vec3 colorWaterMurk;
varying vec3 colorWaterBlue;

varying float iswater;

// Attribute
attribute vec4 mc_Entity;

// Common Variable
const float PI = 3.1415927;

// Program
void main() {

	vec4 position = gl_ModelViewMatrix * gl_Vertex;
	iswater = 0.0f;
	float displacement = 0.0;
	
	posisiMata = position;
	
	vec4 viewpos = gbufferModelViewInverse * position;
	vec3 worldpos = viewpos.xyz + cameraPosition;
	wPos = worldpos;
	worldpos.y -= length(wPos.xz) * 0.2;
	
	if (mc_Entity.x == 8) {
		iswater = 1.0;
		float fy = fract(worldpos.y + 0.001);
		
#ifdef WAVING_WATER
		float wave = 0.05 * sin(2 * PI * (frameTimeCounter * Water_WaveSpeed1 - worldpos.x /  7.0 - worldpos.z / 13.0))
				   + 0.05 * sin(2 * PI * (frameTimeCounter * Water_WaveSpeed2 - worldpos.x / 11.0 - worldpos.z /  5.0));
		displacement = clamp(wave, -fy, 1.0-fy);
	
	#ifdef Dynamic_Water
	float Dynamic_wavingWater = 2.0f;
	#else
	float Dynamic_wavingWater = 1.0f;
	#endif
	
#ifdef Dynamic_Weather
	#ifdef Dynamic_Water
		float next_moon_phase = moonPhase + 1;

		if(float(moonPhase) == 7) {
			next_moon_phase = 0;
		}
		
		float moon_phase_smooth = mix(moonPhase, next_moon_phase, float(worldTime) / 24000.0);
		
		if(hujan < 0.99) {
			Dynamic_wavingWater *= ((abs(float(moon_phase_smooth) - 3) + 1) / 5) + 0.5;
		}
	#endif
#endif
		
		viewpos.y += displacement * 1.0 * Dynamic_wavingWater;
		viewpos.y += displacement * 1.8 * hujan;

#endif
	}
	
	viewpos = gbufferModelView * viewpos;
	gl_Position = gl_ProjectionMatrix * viewpos;
	
	color = gl_Color;
	
	texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).st;
	lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).st;
	
	tangent = vec3(0.0);
	binormal = vec3(0.0);
	normal = normalize(gl_NormalMatrix * normalize(gl_Normal));

	if (gl_Normal.x > 0.5) {
		//  1.0,  0.0,  0.0
		tangent  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0, -1.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	}
	
	else if (gl_Normal.x < -0.5) {
		// -1.0,  0.0,  0.0
		tangent  = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	}
	
	else if (gl_Normal.y > 0.5) {
		//  0.0,  1.0,  0.0
		tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	}
	
	else if (gl_Normal.y < -0.5) {
		//  0.0, -1.0,  0.0
		tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0,  0.0,  1.0));
	}
	
	else if (gl_Normal.z > 0.5) {
		//  0.0,  0.0,  1.0
		tangent  = normalize(gl_NormalMatrix * vec3( 1.0,  0.0,  0.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	}
	
	else if (gl_Normal.z < -0.5) {
		//  0.0,  0.0, -1.0
		tangent  = normalize(gl_NormalMatrix * vec3(-1.0,  0.0,  0.0));
		binormal = normalize(gl_NormalMatrix * vec3( 0.0, -1.0,  0.0));
	}
	
	colorWaterMurk = vec3(0.2f, 0.5f, 0.95f);
	colorWaterBlue = vec3(0.2f, 0.5f, 0.95f);
	colorWaterBlue = mix(colorWaterBlue, vec3(1.0f), vec3(0.5f));
}

#endif